# from  .exceptions  import * 
# from  .point import *
# from  .line  import * 
# from  .ellipse  import * 
# from  .polygon  import * 
# from  .hyperbola  import * 
# from  .triangle  import * 
# from  .rectangle  import * 
# from  .graph  import * 
# from  .delaunay  import * 
# from  .dft import *
# from  .idft import *
# from  .polynomial import *

# p1 = Point(1,1)
# p2 = Point(2,7)

# print(Point.euclideanDistance(p1,p2))


# A =[1,2,3,4]
# print(dft(A))
# print(idft(A))
# print(fpm(A,A))

# from Geom8ry.point import Point 
# from Geom8ry.point import PolarPoint
# # from Geom8ry import * 
# p1 = Point(1,1)
# print(p1)
# pp1 = PolarPoint(2,30)
# p2 = pp1.toCratesian()
# print(p2)
# p3 = Point(2,2)
# # Point()
# print(Point.orientation(p1,p2,p3))
# print(Point.findQuadrant(p2))
# print(Point.euclideanDistance(p1,p2))
# print(Point.manhattanDistance(p1,p2))
# l = [p1,p2,p3]
# print(Point.centroid(l))
# print(Point.convexHull(l))
# # print(Point.closestPairOfPoints(l))
# print(Point.delaunayTriangulation(l))